import argparse
import os
import dlib
from PIL import Image
import numpy as np


def shape_to_np(shape, dtype="int"):
    # Credit from : https://github.com/jrosebr1/imutils/blob/master/imutils/face_utils/helpers.py
    # initialize the list of (x, y)-coordinates
    coords = np.zeros((shape.num_parts, 2), dtype=dtype)
    # loop over all facial landmarks and convert them
    # to a 2-tuple of (x, y)-coordinates
    for i in range(0, shape.num_parts):
        coords[i] = (shape.part(i).x, shape.part(i).y)

    # return the list of (x, y)-coordinates
    return coords


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-d", "--data", help="Folder path containing all the images", required=True)
    parser.add_argument("-o", "--output", help="Output folder to save the results", required=True)
    parser.add_argument("-l", "--landmarks", help="Landmark predictor", default="shape_predictor_68_face_landmarks.dat")
    parser.add_argument("--show", help="Show results on each images", action="store_true")
    args = parser.parse_args()

    # Get the arguments
    dataset_path = args.data
    output_path = args.output
    os.makedirs(output_path, exist_ok=True)
    p = args.landmarks
    show = args.show

    # We need a face detector.  We will use this to get bounding boxes for
    # each face in an image.
    detector = dlib.get_frontal_face_detector()

    # And we also need a shape_predictor.  This is the tool that will predict face
    # landmark positions given an image and face bounding box.  Here we are just
    # loading the model from the shape_predictor_68_face_landmarks.dat file you gave
    # as a command line argument.
    predictor = dlib.shape_predictor(p)

    # Get all files in folder
    img_files = [x for x in os.listdir(dataset_path) if x.split(".")[-1].upper() in ["JPG", "PNG"]]

    for img_file in img_files:

        print(img_file)

        # Load the image and convert to grayscale
        img_file_path = os.path.join(dataset_path, img_file)
        img = Image.open(img_file_path)
        img_gray = np.asarray(img.convert('LA'))[:, :, 0]

        # Find the rectangles containing faces
        rects = detector(img_gray, 0)
        # For each detected face, find the landmark.
        for (i, rect) in enumerate(rects):
            # Make the prediction and transfom it to numpy array
            shape = predictor(img_gray, rect)
            shape_np = shape_to_np(shape)

            # Draw on our image, all the finded cordinate points (x,y)
            if show:
                import matplotlib.pyplot as plt
                fig, ax = plt.subplots(1)
                ax.imshow(img)
                for (x, y) in shape_np:
                    circ = plt.Circle((x, y), 5)
                    ax.add_patch(circ)
                plt.show()

            # Save results
            np.savetxt(os.path.join(output_path, img_file.split(".")[0] + ".txt"), shape_np, fmt="%d")


if __name__ == '__main__':
    main()